package com.ibm.swg.util;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import psdi.mbo.MboRemote;
import psdi.mbo.MboSetRemote;
import psdi.mbo.SqlFormat;
import psdi.util.MXException;

public class PreparedScriptHelper {

	/**
	 * Check that the attributes have only 1 value for each segment.
	 * A case can come when we use linear asset with a dynamic jobplan and we do not want to
	 * use the linear referencement method.
	 * So for the calculation of the with attribute expressions, we must have only 1 value by attribute
	 * @param asset
	 * @param attributes
	 * @return
	 * @throws MXException 
	 * @throws RemoteException 
	 */
	public static boolean verifySegmentAttributesAreUnique(MboRemote asset, Set<String> attributes) throws RemoteException, MXException {
		Set<String> attributesCP = new HashSet<String>();
		for(String s : attributes ) {
			if ( !s.contains(":") ) {
				attributesCP.add(s);
			}
		}
		
		MboSetRemote assetSpecs = asset.getMboSet("ASSETSPEC");
		StringBuilder sb = new StringBuilder();
		for(String attr : attributesCP) {
			sb.append("=").append(attr).append(",");
		}
		sb.deleteCharAt(sb.length() - 1);
		assetSpecs.setQbe("ASSETATTRID", sb.toString());
		
		boolean verif = assetSpecs.count() == attributesCP.size();
		assetSpecs.close();
		
		return verif;
	}
	
	/**
	 * @see PreparedScriptHelper#getSegments(PreparedScript, MboRemote, Double, Double)
	 * @param script
	 * @param asset
	 * @return only one 
	 * @throws RemoteException
	 * @throws MXException
	 */
	public static <T> LinearSegment getSegment(PreparedScript<T> script, MboRemote asset) throws RemoteException, MXException {
		return getSegments(script, asset, null, null, null).get(0);
	}
	
	/**
	 * Get LinearSegments for a linear asset
	 * @param script The script that list the attributes / functions used by the expression
	 * @param asset The linear asset which hold the linear attributes
	 * @param woStartMeasure the start measure of the WO (multiassetlocci start or simulation start)  in which the calculation must be performed. if null the linear segments will be return as a segment (0-1) 
	 * @param woEndMeasure the end measure of the WO (multiassetlocci end or simulation end) in which the calculation must be performed. if null the linear segments will be return as a segment (0-1)
	 * @return an ORDERED collection representing the attribute values by segments for the script
	 */
	public static <T> List<LinearSegment> getSegments(PreparedScript<T> script, MboRemote asset, Double woStartMeasure, Double woEndMeasure, String unitMeasure) throws RemoteException, MXException {
		boolean mustUseMeasure = woStartMeasure != null && woEndMeasure != null;
		
//		SqlFormat sqf;
		StringBuilder sqfWhere;
		List<LinearSegment> segments = new ArrayList<LinearSegment>();
		
		// Loading asset attributes parameter values
		MboSetRemote assetSpecs = asset.getMboSet("ASSETSPEC");
		
		sqfWhere = new StringBuilder("");
		//if ( mustUseMeasure ) {
		//	sqfWhere.append("endmeasure >= :1 and startmeasure <= :2 and ");
		//}
		sqfWhere.append("assetattrid in ( ");
		
		boolean assetAttributeFound = false;
		for(String p : script.getParametersName()) {
//			if ( !script.paramIsFeature(p) && !p.equals(DJPConstants.HOURS_TAG) && !p.equals(DJPConstants.QUANTITY_TAG)) {
			if ( !script.paramIsFeature(p) ) {
				assetAttributeFound = true;
				sqfWhere.append("'").append(SqlFormat.getSQLString(p)).append("',");
			}
				
		}
		sqfWhere.deleteCharAt(sqfWhere.length()-1);
		sqfWhere.append(')');
		
		if ( assetAttributeFound ) {
//			sqf = new SqlFormat(asset.getUserInfo(), sqfWhere.toString());
//			
//			if ( mustUseMeasure ) {
//				sqf.setDouble(1, woStartMeasure);
//				sqf.setDouble(2, woEndMeasure);
//			}
//	
//			assetSpecs.setWhere(sqf.format());
			assetSpecs.setWhere(sqfWhere.toString());
			assetSpecs.setOrderBy("STARTMEASURE, ENDMEASURE asc");
			
			for(MboRemote attr = assetSpecs.moveFirst(); attr != null; attr = assetSpecs.moveNext()) {
				double start = 0;
				double end = 1;
				if ( mustUseMeasure && !attr.isNull("STARTMEASURE") && !attr.isNull("ENDMEASURE") ) {
					start = attr.getDouble("STARTMEASURE");
					end = attr.getDouble("ENDMEASURE");
					String startMeasureUnitId = attr.getString("STARTMEASUREUNITID");
					String endMeasureUnitId = attr.getString("ENDMEASUREUNITID");
					end = DynamicJobPlanHelper.convertValue(attr, end, endMeasureUnitId, unitMeasure);
					start = DynamicJobPlanHelper.convertValue(attr, start, startMeasureUnitId, unitMeasure);
					
					if ( end < woStartMeasure || start > woEndMeasure ) {
						continue;
					}
				}
				segments.add(new LinearSegment( start, end, attr.getString("ASSETATTRID"), attr.getDouble("NUMVALUE")));
			}
			assetSpecs.close();
		}
		// Loading features parameter values
		if ( !script.getFeaturesNames().isEmpty() ) {
			sqfWhere = new StringBuilder("(refobjectid, classstructureid) IN (select assetfeatureid,classstructureid from assetfeature where assetnum = :assetnum and siteid = :siteid ) and refobjectname = 'ASSETFEATURE' ");
//			if ( mustUseMeasure ) {
//				sqfWhere.append(" and endmeasure >= :1 and startmeasure <= :2");
//			}
			sqfWhere.append(" and (feature,assetattrid) in ( ");
			for(String featureName : script.getFeaturesNames()) {
				for(String featureParamName : script.getFeatureParamNames(featureName)) {
					sqfWhere.append("('").append(featureName.toUpperCase()).append("','").append(featureParamName).append("'),");
					
				}
			}
			sqfWhere.deleteCharAt(sqfWhere.length()-1);
			sqfWhere.append(')');
			
//			sqf = new SqlFormat(asset, sqfWhere.toString());
//			if ( mustUseMeasure ) {
//				sqf.setDouble(1, woStartMeasure);
//				sqf.setDouble(2, woEndMeasure);
//			}
//			
//			MboSetRemote featuresAttrSet = asset.getMboSet("$FEATUREATTR","ASSETFEATURESPEC",sqf.format());
			MboSetRemote featuresAttrSet = asset.getMboSet("$FEATUREATTR","ASSETFEATURESPEC", sqfWhere.toString());
			
			featuresAttrSet.setOrderBy("STARTMEASURE, ENDMEASURE asc");
			for(MboRemote featureAttr = featuresAttrSet.moveFirst(); featureAttr != null; featureAttr = featuresAttrSet.moveNext()) {
				double start = 0;
				double end = 1;
				if ( mustUseMeasure && !featureAttr.isNull("STARTMEASURE") && !featureAttr.isNull("ENDMEASURE") ) {
					start = featureAttr.getDouble("STARTMEASURE");
					end = featureAttr.getDouble("ENDMEASURE");
					String startMeasureUnitId = featureAttr.getString("STARTMEASUREUNITID");
					String endMeasureUnitId = featureAttr.getString("ENDMEASUREUNITID");
					end = DynamicJobPlanHelper.convertValue(featureAttr, end, endMeasureUnitId, unitMeasure);
					start = DynamicJobPlanHelper.convertValue(featureAttr, start, startMeasureUnitId, unitMeasure);
					
					if ( end < woStartMeasure || start > woEndMeasure ) {
						continue;
					}
				}
				segments.add(new LinearSegment(start, end, featureAttr.getString("FEATURE") + ":" + featureAttr.getString("ASSETATTRID"), featureAttr.getDouble("NUMVALUE")));
			}
			featuresAttrSet.close();
		}
		segments = LinearSegment.aggregate(segments);
//		Collections.sort(segments);
		
		return segments;
	}
	
	/*
	 * Get LinearSegments for a linear asset
	 * @param script The script that list the attributes / functions used by the expression
	 * @param asset The linear asset which hold the linear attributes
	 * @param woStartMeasure the start measure of the WO (multiassetlocci start or simulation start)  in which the calculation must be performed. if null the linear segments will be return as a segment (0-1) 
	 * @param woEndMeasure the end measure of the WO (multiassetlocci end or simulation end) in which the calculation must be performed. if null the linear segments will be return as a segment (0-1)
	 * @return an ORDERED collection representing the attribute values by segments for the script
	public static <T> List<LinearSegment> getSegments(PreparedScript<T> script, MboRemote asset, Double woStartMeasure, Double woEndMeasure, String unitMeasure) throws RemoteException, MXException {
		boolean mustUseMeasure = woStartMeasure != null && woEndMeasure != null;
		
		SqlFormat sqf;
		StringBuilder sqfWhere;
		List<LinearSegment> segments = new ArrayList<LinearSegment>();
		
		// Loading asset attributes parameter values
		MboSetRemote assetSpecs = asset.getMboSet("ASSETSPEC");
		
		sqfWhere = new StringBuilder("");
		if ( mustUseMeasure ) {
			sqfWhere.append("endmeasure >= :1 and startmeasure <= :2 and ");
		}
		sqfWhere.append("assetattrid in ( ");
		
		boolean assetAttributeFound = false;
		for(String p : script.getParametersName()) {
//			if ( !script.paramIsFeature(p) && !p.equals(DJPConstants.HOURS_TAG) && !p.equals(DJPConstants.QUANTITY_TAG)) {
			if ( !script.paramIsFeature(p) ) {
				assetAttributeFound = true;
				sqfWhere.append('\'').append(SqlFormat.getSQLString(p)).append("',");
			}
				
		}
		sqfWhere.deleteCharAt(sqfWhere.length()-1);
		sqfWhere.append(')');
		
		if ( assetAttributeFound ) {
			sqf = new SqlFormat(asset.getUserInfo(), sqfWhere.toString());
			
			if ( mustUseMeasure ) {
				sqf.setDouble(1, woStartMeasure);
				sqf.setDouble(2, woEndMeasure);
			}
	
			assetSpecs.setWhere(sqf.format());
			assetSpecs.setOrderBy("STARTMEASURE, ENDMEASURE asc");
			
			for(MboRemote attr = assetSpecs.moveFirst(); attr != null; attr = assetSpecs.moveNext()) {
				double start = 0;
				double end = 1;
				if ( mustUseMeasure ) {
					start = attr.getDouble("STARTMEASURE");
					end = attr.getDouble("ENDMEASURE");
				}
				segments.add(new LinearSegment( start, end, attr.getString("ASSETATTRID"), attr.getDouble("NUMVALUE")));
			}
			assetSpecs.close();
		}
		// Loading features parameter values
		if ( !script.getFeaturesNames().isEmpty() ) {
			sqfWhere = new StringBuilder("(refobjectid, classstructureid) IN (select assetfeatureid,classstructureid from assetfeature where assetnum = :assetnum and siteid = :siteid ) and refobjectname = 'ASSETFEATURE' ");
			if ( mustUseMeasure ) {
				sqfWhere.append(" and endmeasure >= :1 and startmeasure <= :2");
			}
			sqfWhere.append(" and (feature,assetattrid) in ( ");
			for(String featureName : script.getFeaturesNames()) {
				for(String featureParamName : script.getFeatureParamNames(featureName)) {
					sqfWhere.append("('").append(featureName.toUpperCase()).append("','").append(featureParamName).append("'),");
					
				}
			}
			sqfWhere.deleteCharAt(sqfWhere.length()-1);
			sqfWhere.append(')');
			
			sqf = new SqlFormat(asset, sqfWhere.toString());
			if ( mustUseMeasure ) {
				sqf.setDouble(1, woStartMeasure);
				sqf.setDouble(2, woEndMeasure);
			}
			
			MboSetRemote featuresAttrSet = asset.getMboSet("$FEATUREATTR","ASSETFEATURESPEC",sqf.format());
			
			featuresAttrSet.setOrderBy("STARTMEASURE, ENDMEASURE asc");
			for(MboRemote featureAttr = featuresAttrSet.moveFirst(); featureAttr != null; featureAttr = featuresAttrSet.moveNext()) {
				double start = 0;
				double end = 1;
				if ( mustUseMeasure ) {
					start = featureAttr.getDouble("STARTMEASURE");
					end = featureAttr.getDouble("ENDMEASURE");
				}
				segments.add(new LinearSegment(start, end, featureAttr.getString("FEATURE") + ":" + featureAttr.getString("ASSETATTRID"), featureAttr.getDouble("NUMVALUE")));
			}
			featuresAttrSet.close();
		}
		segments = LinearSegment.aggregate(segments);
//		Collections.sort(segments);
		
		return segments;
	}
	 */
}
